﻿using System;

namespace Geometry
{
    /// <summary>
    /// anbo
    /// </summary>
    public class LineSegment : IEquatable<LineSegment>
    {

        private readonly int _x1, _x2;

        public LineSegment(int x1, int x2)
        {
            if (x2 < x1)
            {
                throw new ArgumentException(String.Format("x2={0} is smaller than x1={1}", x2, x1));
            }
            _x1 = x1;
            _x2 = x2;
        }

        public int X1 { get { return _x1; } }
        public int X2 { get { return _x2; } }

        public override string ToString()
        {
            return String.Format("LineSegment({0}, {1})", _x1, _x2);
        }

        public bool Contains(int x)
        {
            return _x1 <= x && x <= _x2;
        }

        public bool Contains(LineSegment other)
        {
            if (other == null)
            {
                throw new ArgumentNullException("other");
            }
            return Contains(other.X1) && Contains(other.X2);
        }

        public bool Intersects(LineSegment other)
        {
            if (other == null)
            {
                throw new ArgumentNullException("other");
            }
            return Contains(other.X1) || Contains(other.X2) || other.Contains(_x1) || other.Contains(_x2);
        }

        public bool Equals(LineSegment other)
        {
            if (other == null) return false;
            return _x1 == other._x1 && _x2 == other._x2;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            LineSegment other = obj as LineSegment;
            if (other == null) return false;
            return Equals(other);
        }

        public override int GetHashCode()
        {
            return _x1 + _x2;
        }

        public LineSegment Intersection(LineSegment other)
        {
            if (other == null)
            {
                throw new ArgumentNullException("other");
            }
            int left = Math.Max(_x1, other._x1);
            int right = Math.Min(_x2, other._x2);
            if (left > right) return null;
            return new LineSegment(left, right);
        }
    }
}
