﻿using System;

namespace Geometry
{
    /// <summary>
    /// anbo
    /// </summary>
    public class LineSegmentDouble : IEquatable<LineSegmentDouble>
    {

        private readonly double _x1, _x2;
        private readonly double Delta = 0.00001;

        public LineSegmentDouble(double x1, double x2)
        {
            if (x2 < x1)
            {
                throw new ArgumentException(String.Format("x2={0} is smaller than x1={1}", x2, x1));
            }
            _x1 = x1;
            _x2 = x2;
        }

        public double X1 { get { return _x1; } }
        public double X2 { get { return _x2; } }

        public override string ToString()
        {
            return String.Format("LineSegmentDouble({0}, {1})", _x1, _x2);
        }

        public bool Contains(double x)
        {
            return _x1 <= x && x <= _x2;
        }

        public bool Contains(LineSegmentDouble other)
        {
            return Contains(other.X1) && Contains(other.X2);
        }

        public bool Intersects(LineSegmentDouble other)
        {
            return Contains(other.X1) || Contains(other.X2) || other.Contains(_x1) || other.Contains(_x2);
        }

        public bool Equals(LineSegmentDouble other)
        {

            if (other == null) return false;
            return Math.Abs(_x1 - other._x1) < Delta && Math.Abs(_x2 - other._x2) < Delta;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            LineSegmentDouble other = obj as LineSegmentDouble;
            if (other == null) return false;
            return Equals(other);
        }

        public override int GetHashCode()
        {
            return _x1.GetHashCode() + _x2.GetHashCode();
        }

        public LineSegmentDouble Intersection(LineSegmentDouble other)
        {
            double left = Math.Max(_x1, other._x1);
            double right = Math.Min(_x2, other._x2);
            if (left > right) return null;
            return new LineSegmentDouble(left, right);
        }
    }
}
