﻿using System;
using Geometry;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GeometryTest
{
    [TestClass]
    public class LineSegmentDoubleTest
    {
        private double Delta = 0.00001;

        [TestMethod]
        public void TestConstructor()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 1);
            Assert.AreEqual(1.0, segment.X1, Delta);

            LineSegmentDouble segment2 = new LineSegmentDouble(1, 120);
            try
            {
                new LineSegmentDouble(1, 0);
                Assert.Fail();
            }
            catch (ArgumentException ex)
            {
                Assert.AreEqual("x2=0 is smaller than x1=1", ex.Message);
            }
        }

        [TestMethod]
        public void TestToString()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            Assert.AreEqual("LineSegmentDouble(1, 10)", segment.ToString());
        }

        [TestMethod]
        public void TestContains1()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            Assert.IsFalse(segment.Contains(0));
            Assert.IsTrue(segment.Contains(1));
            Assert.IsTrue(segment.Contains(5));
            Assert.IsTrue(segment.Contains(10));
            Assert.IsFalse(segment.Contains(11));

        }

        [TestMethod]
        public void TestContains2()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            Assert.IsTrue(segment.Contains(segment));
            LineSegmentDouble segment2 = new LineSegmentDouble(2, 8);
            Assert.IsTrue(segment.Contains(segment2));
            Assert.IsFalse(segment2.Contains(segment));
            LineSegmentDouble segment3 = new LineSegmentDouble(5, 11);
            Assert.IsFalse(segment.Contains(segment3));
            Assert.IsFalse(segment3.Contains(segment2));
        }

        [TestMethod]
        public void TestInsersects()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            Assert.IsTrue(segment.Intersects(segment));
            LineSegmentDouble segment2 = new LineSegmentDouble(2, 8);
            Assert.IsTrue(segment.Intersects(segment2));
            Assert.IsTrue(segment2.Intersects(segment));
            LineSegmentDouble segment3 = new LineSegmentDouble(5, 11);
            Assert.IsTrue(segment.Intersects(segment3));
            Assert.IsTrue(segment3.Intersects(segment2));

            LineSegmentDouble segment4 = new LineSegmentDouble(11, 88);
            Assert.IsFalse(segment.Intersects(segment4));
        }

        [TestMethod]
        public void TestEquals()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            Assert.IsTrue(segment.Equals(segment));
            LineSegment segment2 = new LineSegment(1, 11);
            Assert.IsFalse(segment.Equals(segment2));
            Assert.IsFalse(segment.Equals(null));

            Assert.IsFalse(segment.Equals("somestring"));
            string str = null;
            Assert.IsFalse(segment.Equals(str));

            object obj = new LineSegmentDouble(2, 8);
            Assert.IsFalse(segment.Equals(obj));
        }

        [TestMethod]
        public void TestIntersection()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            LineSegmentDouble segment2 = new LineSegmentDouble(2, 11);
            LineSegmentDouble segment3 = segment.Intersection(segment2);
            Assert.AreEqual(2.0, segment3.X1, Delta);
            Assert.AreEqual(10.0, segment3.X2, Delta);

            LineSegmentDouble segment4 = new LineSegmentDouble(11, 15);

            LineSegmentDouble segment5 = segment2.Intersection(segment4);
            Assert.AreEqual(11, segment5.X1); 
            Assert.AreEqual(11, segment5.X2);

            Assert.IsNull(segment.Intersection(segment4));
        }

        [TestMethod]
        public void TestGetHashCode()
        {
            LineSegmentDouble segment = new LineSegmentDouble(1, 10);
            LineSegmentDouble segment2 = new LineSegmentDouble(1, 10);
            Assert.AreEqual(segment.GetHashCode(), segment2.GetHashCode());
        }
    }
}
