﻿using System;
using Geometry;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GeometryTest
{
    [TestClass]
    public class LineSegmentTest
    {
        [TestMethod]
        public void TestConstructor()
        {
            LineSegment segment = new LineSegment(1, 1);
            LineSegment segment2 = new LineSegment(1, 120);
            try
            {
                new LineSegment(1, 0);
                Assert.Fail();
            }
            catch (ArgumentException ex)
            {
                Assert.AreEqual("x2=0 is smaller than x1=1", ex.Message);
            }
        }

        [TestMethod]
        public void TestToString()
        {
            LineSegment segment = new LineSegment(1, 10);
            Assert.AreEqual("LineSegment(1, 10)", segment.ToString());
        }

        [TestMethod]
        public void TestContains1()
        {
            LineSegment segment = new LineSegment(1, 10);
            Assert.IsFalse(segment.Contains(0));
            Assert.IsTrue(segment.Contains(1));
            Assert.IsTrue(segment.Contains(5));
            Assert.IsTrue(segment.Contains(10));
            Assert.IsFalse(segment.Contains(11));
        }

        [TestMethod]
        public void TestContains2()
        {
            LineSegment segment = new LineSegment(1, 10);
            Assert.IsTrue(segment.Contains(segment));
            LineSegment segment2 = new LineSegment(2, 8);
            Assert.IsTrue(segment.Contains(segment2));
            Assert.IsFalse(segment2.Contains(segment));
            LineSegment segment3 = new LineSegment(5, 11);
            Assert.IsFalse(segment.Contains(segment3));
            Assert.IsFalse(segment3.Contains(segment2));
            try
            {
                segment.Contains(null);
                Assert.Fail();
            }
            catch (ArgumentNullException ex)
            {
                Assert.AreEqual("other", ex.ParamName);
            }
        }

        [TestMethod]
        public void TestInsersects()
        {
            LineSegment segment = new LineSegment(1, 10);
            Assert.IsTrue(segment.Intersects(segment));
            LineSegment segment2 = new LineSegment(2, 8);
            Assert.IsTrue(segment.Intersects(segment2));
            Assert.IsTrue(segment2.Intersects(segment));
            LineSegment segment3 = new LineSegment(5, 11);
            Assert.IsTrue(segment.Intersects(segment3));
            Assert.IsTrue(segment3.Intersects(segment2));

            LineSegment segment4 = new LineSegment(11, 88);
            Assert.IsFalse(segment.Intersects(segment4));

            try
            {
                segment.Intersects(null);
                Assert.Fail();
            }
            catch (ArgumentNullException ex)
            {
                Assert.AreEqual("other", ex.ParamName);
            }
        }

        [TestMethod]
        public void TestEquals()
        {
            LineSegment segment = new LineSegment(1, 10);
            Assert.IsTrue(segment.Equals(segment));
            LineSegment segment2 = new LineSegment(1, 11);
            Assert.IsFalse(segment.Equals(segment2));
            Assert.IsFalse(segment.Equals(null));

            Assert.IsFalse(segment.Equals("somestring"));
            string str = null;
            Assert.IsFalse(segment.Equals(str));

            object obj = new LineSegment(2, 8);
            Assert.IsFalse(segment.Equals(obj));
        }

        [TestMethod]
        public void TestIntersection()
        {
            LineSegment segment = new LineSegment(1, 10);
            LineSegment segment2 = new LineSegment(2, 11);
            LineSegment segment3 = segment.Intersection(segment2);
            Assert.AreEqual(2, segment3.X1);
            Assert.AreEqual(10, segment3.X2);

            LineSegment segment4 = new LineSegment(11, 15);

            LineSegment segment5 = segment2.Intersection(segment4);
            Assert.AreEqual(11, segment5.X1); 
            Assert.AreEqual(11, segment5.X2);

            Assert.IsNull(segment.Intersection(segment4));

            try
            {
                segment.Intersection(null);
                Assert.Fail();
            }
            catch (ArgumentNullException ex)
            {
                Assert.AreEqual("other", ex.ParamName);
            }
        }

        [TestMethod]
        public void TestGetHashCode()
        {
            LineSegment segment = new LineSegment(1, 10);
            LineSegment segment2 = new LineSegment(1, 10);
            Assert.AreEqual(segment.GetHashCode(), segment2.GetHashCode());
        }
    }
}
